import os
import math

import gzip
import shutil

import requests
from clint.textui import progress
from mnist import MNIST

url_base = 'http://yann.lecun.com/exdb/mnist/'

filenames = (
    'train-images-idx3-ubyte',
    'train-labels-idx1-ubyte',
    't10k-images-idx3-ubyte',
    't10k-labels-idx1-ubyte',
)


def is_downloaded(path):
    for name in filenames:
        if not os.path.isfile(os.path.join(path, name)):
            return False
    return True


def save_file(src, dest, silent):
    r = requests.get(src, stream=True)

    with open(dest, 'wb') as f:
        chunks = r.iter_content(chunk_size=1024)
        if not silent:
            chunk_count = int(math.ceil(int(r.headers.get('content-length')) / 1024.0))
            chunks = progress.bar(chunks, expected_size=chunk_count)

        for chunk in chunks:
            if chunk:
                f.write(chunk)


def download(dest, silent):
    for name in filenames:
        path = os.path.join(dest, name)
        gzname = name + '.gz'
        gzpath = os.path.join(dest, gzname)

        if not silent:
            print 'Downloading %s' % name
        save_file(url_base + gzname, gzpath, silent)

        with gzip.open(gzpath, 'rb') as f_in, open(path, 'wb') as f_out:
            shutil.copyfileobj(f_in, f_out)
            os.unlink(gzpath)


def ensure_files_exist(path, silent):
    if not os.path.isdir(path):
        if os.path.isfile(path):
            raise IOError('%s already exists but is a file' % path)
        os.mkdir(path)

    if not is_downloaded(path):
        download(path, silent)


def load(silent=False):
    ensure_files_exist('./.mnist', silent)

    mnist = MNIST('./.mnist')
    return mnist.load_training(), mnist.load_testing()
